c-----------------------------------------------------------------------
      program append
c-----------------------------------------------------------------------
c
c --- APPEND     Version: 7.0.0      Level:  141010                MAIN
c 
c     Copyright (c) 1999-2014 by Exponent, Inc.
c
c --- PURPOSE: Append two or more CALPUFF concentration, flux, or 
c              relative humidity (visibility) files with options 
c              to skip periods at the beginning of each input file
c
c --- Written by:  F. Robe
c
c --- UPDATES:
c
c --- Version TNG-3.0.0 to Version 7.0.0, L141010
c     (1) Trap and report error opening files (not found)
c      NEW:       OPEN_ERR
c      Modified:  (main)
c     (2) Implement SYSTEM 7 version notation (System.Update.BugFix)
c     (3) Recognize CALPUFF.DAT dataset TNG-3.0  =  dataset 7.0
c      Modified:  RDWR1
c
c --- Version 2.36, L110301 to Version TNG-3.0.0, L140913
c     (1) Added CALPUFF.DAT dataset version TNG-3.0
c         -Header record NCOM+3 replaces number of sources of each type
c          with the number of types (new types may be added)
c         -Header record NCOM+3 has number of discrete receptor-groups
c         -Header record NCOM+3a contains the numbers of each type
c         -Header record NCOM+6 contains discrete receptor pole-ht and
c          receptor-group name index
c         -Header record NCOM+6a contains discrete receptor-group names
c         -Header record NCOM+9(plus) contains the source names
c          (not written to VISIBILITY files)
c     (2) Expand APPEND version string to 12 characters
c      Modified:  PARAMS.APP
c                 (main), RDWR1
c
c --- Version 2.351, L090511 to Version 2.36, L110301
c --- Modified by D. Strimaitis
c     (1) Add checks to demand that species in all files are in the same
c         order and have the same units (e.g., CALPUFF.DAT Dataset v2.2)
c      Modified:  RDWR1
c
c --- Version 2.35, L070117 to Version 2.351, L090511
c --- Modified by D. Strimaitis
c     (1) Add subroutines to convert time zone formats for reporting
c         to the list file
c      New     :  BASRUTC, UTCBASR
c      Modified:  RDWR1
c
c --- Version 2.34, L051122 to Version 2.35, L070117
c --- Modified by D. Strimaitis
c     (1) Added CALPUFF.DAT dataset version 2.2
c               Add header record after species-level list for
c                 units of each species-level
c               Base time zone is character*8 (e.g. UTC-0500)
c      Modified:  RDWR1
c
c --- Version 2.33, L040923 to Version 2.34, L051122
c --- Modified by D. Strimaitis
c     (1) Changed language written to list file to remove "hour"
c         terminology because APPEND can be applied to CALPUFF output
c         files with non-hourly averaging periods
c      Modified:  MAIN, RDWR1, RDWRHR
c     (2) Compute start time of first output period using actual 
c         averaging time (do not assume hourly)
c      Modified:  RDWR1
c
c --- Version 2.32, L040623 to Version 2.33, L040923
c --- Modified by D. Strimaitis
c     (1) Bug-fix: Date-time written to the output file header for the
c         start of the data is the starting date-time of the first file
c         appended, regardless of whether hours are skipped at the start
c         of this file.
c      Modified:  RDWR1
c
c --- Version 2.31, L040615 to Version 2.32, L040623
c --- Modified by D. Strimaitis
c     (1) Bug-fix: logic for storing source names is faulty if more than
c         one source type is in file (APPEND fails QA check)
c      Modified:  RDWR1
c
c --- Version 2.3, L031017 to Version 2.31, L040615
c --- Modified by D. Strimaitis
c     (1) Bug-fix: number of periods in appended file was not updated
c         to NTOTAL in dataset version 2.1 processing
c      Modified:  RDWR1
c
c --- Version 2.2, L030402 to Version 2.3, L031017
c --- Modified by D. Strimaitis
c     (1) Changes to read DATASET 2.1 CALPUFF output files
c      Modified:  PARAMS.APP, RDWR1, RDWRHR
c      Removed:   RDHR
c
c --- Version 2.0, L020401 to Version 2.2, L030402
c --- Modified by F. Robe, D. Strimaitis
c     ** Requires DATASET 2.0 CALPUFF output format **
c     (1) Changes to read NOOBS CALPUFF files (FRR)
c      -  Additional parameter I2DRH (flag for 2D arrays of rel.hum)
c      -  2-D arrays of relative humidity
c      Changes to subroutines:
c        RDWR1 : add i2drh flag in header record #1 and define xmet,ymet
c                differentiate headers for RH file
c        RDWR  : add i2drh flag in header record #1 and define xmet,ymet
c                differentiate headers for RH file
c                2D array of RH if i2drh=1
c        RDWRHR: 2D array of RH if i2drh=1
c
c     (2) Changes to read/write extended header records (DGS)
c      -  CALPUFF control file documentation from first file retained
c         in header of output file
c      -  /MAP/ variables added
c    
c --- Version 1.0, L961202 (MERGE2) to Version 2.0, L990501 (APPEND)
c --- Modified by J. Scire (5/99)
c     (1) Allow processing of relative humdity files in addition to
c         concentration and flux files
c     (2) Standardize file naming conventions
c     (3) Add QA checks to protect against exceeding array dimensions
c     (4) Remove parameter and array variables not used by the program
c     (5) Add Version and Level numbers to code and output
c
c --- Modified made from MERGE.for on 6/30/97 by Zhong-Xiang Wu
c     (1) The output file is a plan append of all merged files except 
c         overlap part, or, more precisely, in the same form of 
c         compression; that is, the output file may or may not be 
c         compressed depending on the source files. No uncompress is 
c         done if the input file is compressed.
c     (2) No extraction for any part of sampling domain.
c
c --- Revisions on 3/29/97 by Zhong-Xiang Wu
c     (1) Add hours need to be skipped at spin-up time and total hours
c         in a merged file in append2.inp file, and make necessary 
c         changes in the program.
c     (2) Use logical array lcompr(mxfil) to enable the program to read
c         the compressed data on different platforms.
c
c-----------------------------------------------------------------------
c --- Include parameters
      include 'params.app'
c
      character*80 titleo(3)
      character*70 cfile(mxfil),cfilout
      character*12 cver
      character*8  clevel
      dimension nskip(mxfil),nhrs(mxfil)

      logical lsamp,lend,lcompr,lskip
c
c --- Open control & list files
      open(io5,file='append.inp',status='old',iostat=ierr)
      if(ierr.NE.0) call OPEN_ERR(-1,'(main)','CONTROL File',
     &                            'append.inp',io5)
      open(io6,file='append.lst',status='unknown')
c
c --- Set Version and Level numbers
      cver='7.0.0'
      clevel='141010'
c
c ----------------------------
c --- Read control file inputs
c ----------------------------
c
c --- Read the file type (1=conc/flux files, 2=rh/visibility files)
      read(io5,*)itype
      if(itype.ne.1.and.itype.ne.2)then
         write(io6,*)
         write(io6,*)
         write(io6,*)
         write(io6,*)'ERROR -- Invalid file type'
         write(io6,*)'ITYPE = ',itype
         write(io6,*)'Valid values are 1 (conc/flux files) ',
     1               'or 2 (rh/visibility files)'
         stop 'APPEND Halted --- See List File'
      endif
c --- Read the number of files to append (NFILES)
      read(io5,*)nfiles
      if(nfiles.gt.mxfil)then
         write(io6,*)
         write(io6,*)
         write(io6,*)
         write(io6,*)'ERROR - too many files for array dimensions -- ',
     1   'NFILES = ',nfiles,' MXFIL = ',mxfil
                  stop 'APPEND Halted --- See List File'
      endif
c --- Read input file names (CFILE(mxfil))
      ntotal=0 
      do i=1,nfiles
         read(io5,'(a70)')cfile(i)
         read(io5,*)nskip(i),nhrs(i)      ! added by Wu
         ntotal=ntotal+nhrs(i)-nskip(i)   ! total periods for merge
      enddo
c --- Read output file name (CFILOUT)
      read(io5,'(a70)')cfilout
c --- Read the 3-line title of the new, summed file (TITLEO(3))
      do i=1,3
         read(io5,'(a80)')titleo(i)
      enddo

c
c -----------------------------------------
c --- Echo back the inputs to the list file
c -----------------------------------------
      write(io6,1400)cver,clevel
1400  format(1x,'APPEND -- Version: ',a12,3x,'Level: ',a8/)
c
      write(io6,*)' Number of input files (NFILES) = ',nfiles
      write(io6,*)
      write(io6,*)' Names of input files: '
      write(io6,*)
      do i=1,nfiles
         write(io6,10)i,cfile(i)
10       format(5x,'No. ',i3,3x,'Filename: ',a70)
      enddo
      write(io6,*)
      write(io6,12)cfilout
12    format(/1x,'Output Filename: ',a70)
      write(io6,*)
      write(io6,*)' Title lines on new output file:'
      do i=1,3
         write(io6,18)titleo(i)
18       format(5x,a80)
      enddo

c --- Transfer control file image to scratch file
      call TFERCF(titleo,ncommout)

c
c --- Open the input files
      do i=1,nfiles
         io=io21+i-1
         open(io,file=cfile(i),status='old',form='unformatted',
     &        iostat=ierr)
         if(ierr.NE.0) call OPEN_ERR(io6,'(main)','MODEL File',
     &                               cfile(i),io)
      enddo
c
c --- Open the output file
      open(io10,file=cfilout,status='unknown',form='unformatted')
c
c --- Read and write the header records
      ns=nskip(1)
c frr (4/02) initialize 2D-RH flag
      i2drh=0
      call rdwr1(cver,clevel,ncommout,titleo,nfiles,ntotal,ns,itype,
     &           nspeci,nreci,nctreci,lsamp,nxsam,nysam,iflux,lcompr,
     &           isrcinfo,isrsindv,nsrc,itime,nsstai,nxmet,nymet,i2drh)
c
c --- Loop over the files:
      icount=0
      write(io6,*)
      do i=1,nfiles
         io=io21+i-1
         ns=nskip(i)
         nhr=nhrs(i)
         nproc=nhr-ns
 
         print *,' File: ',i,' Skipping: ',ns,' Writing: ',nproc
         write(io6,*)' File: ',i,' Skipping: ',ns,' Writing: ',nproc
c
         if(ns.ge.1) then
            lskip=.TRUE.
            do j=1,ns
               call rdwrhr(io,lskip,nspeci,nreci,nctreci,lsamp,
     1              nxsam,nysam,iflux,lcompr,i,itype,isrcinfo,
     2              isrsindv,nsrc,itime,nsstai,nxmet,nymet,i2drh,lend)
            enddo
         endif          

c ------ Loop over all periods needed in the file for merge
      
         lskip=.FALSE.
         do j=1,nproc
             call rdwrhr(io,lskip,nspeci,nreci,nctreci,lsamp,
     1            nxsam,nysam,iflux,lcompr,i,itype,isrcinfo,
     2            isrsindv,nsrc,itime,nsstai,nxmet,nymet,i2drh,lend)
            if(lend)go to 999
            icount=icount+1
         enddo
999      continue
      enddo 
c
      write(io6,*)
      write(io6,*)'Number of periods processed: ',icount
      print *,'Number of periods processed: ',icount
c
      stop
      end

c----------------------------------------------------------------------
      subroutine rdwr1(cver,clev,ncommout,titleo,nfiles,ntotal,nskip,
     &   itype,nspeci,nreci,nctreci,lsamp,nxsam,nysam,iflux,lcompr_,
     &   isrcinfo_,isrcindv_,nsrc_,iptime_,nsstai,nxmet,nymet,i2drh)
c----------------------------------------------------------------------
c
c --- APPEND     Version: 7.0.0      Level:  141010              RDWR1
c                J. Scire
c
c --- PURPOSE:  Read & write the header records of the CALPUFF
c               output files (concentration, dry flux, wet flux,
c               and relative humidity files)
c               
c --- UPDATE
c --- 7.0.0(141010) from TNG-3.0.0(140913) (DGS)
c                    - CALPUFF.DAT dataset version 7.0 is same as
c                      TNG-3.0 format
c --- TNG-3.0.0(140913) from V2.36(110301) (DGS)
c                    - CALPUFF.DAT dataset version TNG-3.0 format
c                    - All files must be same dataset version
c --- V2.36(110301) from V2.351(090511) (DGS)
c                    - Add check for matching units and species
c --- V2.351(090511) from V2.35(070117) (DGS)
c                    - Convert and report time zone formats
c --- V2.35(070117) from V2.34(051122) (DGS)
c                    - CALPUFF.DAT dataset version 2.2 format
c                      Add header record after species-level list for
c                        units of each species-level
c                      Base time zone read as character*8
c                       (e.g. UTC-0500)
c --- V2.34(051122) from V2.33(040923) (DGS)
c                    - Remove "hour" terminology
c                    - Compute start time of first output period using
c                      actual averaging time (do not assume hourly)
c --- V2.33(040923) from V2.32(040623) (DGS)
c                    - Recompute the starting date-time if hours are
c                      skipped at the start of the first file (add
c                      nskip to argument list)
c --- V2.32(040623) from V2.31(040615) (DGS)
c                    - Version 2.1 format bug fixed:  logic for
c                      storing source names is faulty if more than
c                      one source type is in file (APPEND fails
c                      QA check)
c --- V2.31(040615) from V2.3 (031017) (DGS)
c                    - Version 2.1 format bug fixed:  NTOTAL from
c                      arg list replaces irlg() from headers when
c                      writing output header
c --- V2.3 (031017) from V2.2 (030402) (DGS)
c                    - CALPUFF.DAT dataset version 2.1 format
c                    - Require all files be either compressed or
c                      uncompressed
c --- V2.2 (030402) from V2.0 (020401) (DGS)
c                    - CALPUFF control file documentation from first
c                      file retained in header of output file
c                    - CALPUFF.DAT dataset version 2.0 format
c---- Modifications: - FRR(4/02) add 2D RH array flag
c                    - Adds up the lengths of each run and writes out
c                      the total length in the output header
c                    - Does not add up the sources any longer
c          Modified by J. Scire, 5/99
c                    - Add capability to read RH files (5/99)
c          Modified by F.Robe, Earth Tech, 961112.
c
c --- INPUTS:
c                 CVER - Char.*12    - APPEND version
c                 CLEV - Char.*8     - APPEND level
c             NCOMMOUT - integer     - Number of comment records saved
c                                      for output header
c            TITLEO(3) - Char.*80    - New run title (3 lines of
c                                      80 char. each) that will be
c                                      output to the new, summed file
c               NFILES - integer     - Number of files to be summed
c               NTOTAL - integer     - Total number of periods in the
c                                      appended output file (does not
c                                      count periods skipped by user
c                                      request)
c                NSKIP - integer     - Number of periods skipped at start
c                                      of file 1
c                ITYPE - integer     - File type:
c                                         1 = concentration/flux files
c                                         2 = relative humidity files
c       Parameters:
c             MXNX, MXNY, MXREC, MXRECT, MXSPEC, MXFIL, MXSS,
c             MXSCRTYPE, MXRGRP,
c             IO6, IO10, IO21
c
c --- OUTPUT:
c               NSPECI - integer     - Number of species in files
c                NRECI - integer     - Number of discrete receptors
c              NCTRECI - integer     - Number of special complex terrain
c                LSAMP - logical     - Flag indicating if gridded
c                                      receptors used (T=yes, F=no)
c                                      (CTSG) receptors
c                NXSAM - integer     - No. cells in X direction of
c                                      gridded receptor grid
c                NYSAM - integer     - No. cells in Y direction of
c                                      gridded receptor grid
c                IFLUX - integer     - Flag indicating if file are
c                                      concentrations (IFLUX=0) or
c                                      wet/dry fluxes (IFLUX=1).  The
c                                      CTSG receptors have no flux data
c              LCOMPR_ - logical     - Data compression flag
c            ISRCINFO_ - integer     - Source information flag
c            ISRCINDV_ - integer     - Individual source contribution
c                                      flag
c                NSRC_ - integer     - Number of sources
c              IPTIME_ - integer     - Time format flag
c               NSSTAI - integer     - Number of surface (RH) stations
c                                      in visibility file
c               NXMET - integer      - Actual x dimension of met grid
c               NYMET - integer      - Actual y dimension of met grid
c               I2DRH - integer      - Flag indicating if 2D array of RH
c
c --- RDWR1 called by:  MAIN
c --- RDWR1 calls:      INCR, UTCBASR, BASRUTC
c
c----------------------------------------------------------------------
c
c --- Include parameters
      include 'params.app'
c
      real dxkm(mxfil),dykm(mxfil),xorigkm(mxfil),yorigkm(mxfil)
      real xrec(mxrec,mxfil),yrec(mxrec,mxfil),zrec(mxrec,mxfil)
      real htrec(mxrec,mxfil)
      real xrct(mxrect,mxfil),yrct(mxrect,mxfil),zrct(mxrect,mxfil)
      real xkmsta(mxss,mxfil),ykmsta(mxss,mxfil)

      integer ibyr(mxfil),ibjul(mxfil),ibhr(mxfil),irlg(mxfil),
     1 iavg(mxfil),nx(mxfil),ny(mxfil),ione(mxfil),nssta(mxfil),
     2 ibcomp(mxfil),iecomp(mxfil),jbcomp(mxfil),jecomp(mxfil),
     3 ibsamp(mxfil),jbsamp(mxfil),iesamp(mxfil),jesamp(mxfil),
     4 meshdn(mxfil),nrec(mxfil),nctrec(mxfil),nspout(mxfil),
     6 i2dmet(mxfil),irgrp(mxrec,mxfil),nrgrp(mxfil)

      character*80 argrpnam(mxrgrp,mxfil)

c --- Variables added/modified for DATASET VERSION 7.0
c ---                         (DATASET VERSION TNG-3.0)
c     NSRCTYPE(mxfil)Number of source types in dataset
c     NSTOT          Total number of sources in a file
c     NSTYPE(mxfil,mxsrctype) holds the number of each type
c                     of source in each file
c               1 - point sources (Control File)
c               2 - point sources (Var. Emissions File)
c               3 - area sources (Control File)
c               4 - area sources (Var. Emissions File)
c               5 - line sources (Control File)
c               6 - line sources (Var. Emissions File)
c               7 - volume sources (Control File)
c               8 - volume sources (Var. Emissions File)
c               9 - boundary puffs (always 0)
c              10 - flare sources (Control File)
c              11 - flare sources (Var. Emissions File)
c              12 - road sources (Control File)
c              13 - road sources (Var. Emissions File)
c    HTREC(mxrec,mxfil) Ht of discrete receptor (mAGL)
c    IRGRP(mxrec,mxfil) Discrete receptor-group index
c    NRGRP(mxfil)       Number of Discrete receptor-groups
c    ARGRPNAM(mxrgrp,mxfil) Discrete receptor-group names
c                       (Same in all files)
cc --- Variables added for DATASET VERSION 2.2
c      character*8 abtz(mxfil)
c      character*16 aspunit(mxspec,mxfil)
c      logical lspunit(mxfil)
c
cc --- Variables added for DATASET VERSION 2.1
c      real xbtz(mxfil)
c      integer ibsec(mxfil),nsecdt(mxfil),npt1(mxfil),npt2(mxfil),
c     1 nar1(mxfil),nar2(mxfil),nln1(mxfil),nln2(mxfil),nvl1(mxfil),
c     2 nvl2(mxfil),msource(mxfil)
c      integer iptime(mxfil),isrcinfo(mxfil),isrcindv(mxfil),nsrc(mxfil)
c      character*16 csource(mxsrc)


      real xbtz(mxfil)
      integer ibsec(mxfil),nsecdt(mxfil),nstype(mxfil,mxsrctype),
     &        msource(mxfil),nsrc(mxfil),nsrctype(mxfil)
      integer nstot
      integer iptime(mxfil),isrcinfo(mxfil),isrcindv(mxfil)

      character*16 aspunit(mxspec,mxfil)
      character*16 csource(mxsrc)
      character*8 abtz(mxfil)
      character*4 cstype(mxsrctype)
      integer ihill(mxrect,mxfil)

c --- Keep NEARS array as 2-D rather than (-,-,mxfil) because data
c --- from only one file is written to the output file
      integer nears(mxnx,mxny)

      character*12 cver
      character*8  clev
      character*80 titleo(3)
      character*80 title(3,mxfil)
      character*15 csout(mxspec,mxfil)
      character*15 cname1(mxfil),cname2(mxfil),cnamen
      character*12 cmodel(mxfil),ver(mxfil),level(mxfil)

      logical lspunit(mxfil)
      logical lsgrid(mxfil),lsamp,lcompr(mxfil),lcompr_,lfatal

c --- Arrays for mapping parameters
      character*4 utmhem(mxfil)
      character*8 datum(mxfil),pmap(mxfil)
      character*12 daten(mxfil)
      character*16 clat0(mxfil),clon0(mxfil),clat1(mxfil),clat2(mxfil)
      integer iutmzn(mxfil)
      real feast(mxfil),fnorth(mxfil)
      real rnlat0(mxfil),relon0(mxfil),xlat1(mxfil),xlat2(mxfil)

c --- Local Variables for header text
      character*16 dataset,dataver
      character*33 blank33,break33
      character*64 datamod
      character*132 doc1,comment1,blank,break

      data blank33/'                                 '/
      data break33/'.................................'/

c --- Configure output documentation
      data doc1/'Produced by APPEND Version: '/

c --- Set names of each source type for report section
c --- (MXSRCTYPE values)
      data cstype/'pt1 ','pt2 ','ar1 ','ar2 ',
     &            'ln1 ','ln2 ','vl1 ','vl2 ','bcon',
     &            'fl1 ','fl2 ','rd1 ','rd2 '/

c --- Set blank (132 characters)
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Set break (132 characters)
      break(1:33)=break33
      break(34:66)=break33
      break(67:99)=break33
      break(100:132)=break33

c --- Initialize number of sources (arrays)
      nsrc=0
c --- Initialize number of receptor-groups (array)
      nrgrp=0

c --- Construct the version-level comment string
      j=29
      do i=1,12
         if(cver(i:i).NE.' ') then
            doc1(j:j)=cver(i:i)
            j=j+1
         endif
      enddo
      j=j+1
      comment1(j:j+7)=' Level: '
      j=j+8
      do i=1,8
         if(clev(i:i).NE.' ') then
            doc1(j:j)=clev(i:i)
            j=j+1
         endif
      enddo
c
c ----------------------------------------------------------------------
c --- INPUT header record data from input files
c ----------------------------------------------------------------------
c
c --- Read all of the information from "NFILES" CALPUFF files
      do i=1,nfiles
         io=io21+i-1

c ---    Set flag for specie-level units (introduced in Dataset 2.2)
         lspunit(i)=.FALSE.

c ---    Read and test first record to check header format
c ---    Record #1 - File Declaration -- 24 words
         read(io) dataset,dataver,datamod
         ifilver=0
         if(dataset.EQ.'CONC.DAT') then
            ifilver=1
         elseif(dataset.EQ.'DFLX.DAT') then
            ifilver=1
         elseif(dataset.EQ.'WFLX.DAT') then
            ifilver=1
         elseif(dataset.EQ.'VISB.DAT') then
            ifilver=1
         endif

         if(ifilver.EQ.0) then
c ---       File format is not supported
            write(*,*)'Header records indicate unsupported format'
            write(*,*)'for CALPUFF data file!  FILE = ',i
            write(*,*)'Expected datset version 2.0 or later'
            stop
         endif
         if(i.EQ.1) then
c ---       To scratch file
            write(iox,'(a132)') blank
            write(iox,'(a132)') break
            write(iox,'(a132)') blank
            write(iox,'(2a16,a64)') dataset,dataver,datamod
            ncommout=ncommout+4
c ---       Number of comment records
            read(io) ncom
            ncommout=ncommout+ncom
            do k=1,ncom
               comment1=blank
               read(io) comment1
               write(iox,'(a132)') comment1
            enddo
         else
c ---       Skip over comment records
            read(io) ncom
            do k=1,ncom
               read(io) comment1
            enddo
         endif
c
c ---    HEADER RECORD NCOM+3 -- General run parameters
c
         if(dataver.EQ.'2.0             ') then
           read(io)cmodel(i),ver(i),level(i),ibyr(i),ibjul(i),ibhr(i),
     1       irlg(i),iavg(i),nx(i),ny(i),dxkm(i),dykm(i),ione(i),
     2       xorigkm(i),yorigkm(i),nssta(i),ibcomp(i),iecomp(i),
     3       jbcomp(i),jecomp(i),ibsamp(i),jbsamp(i),iesamp(i),
     4       jesamp(i),meshdn(i),
     5       nstype(i,1),nstype(i,3),nstype(i,5),nstype(i,7),
     6       nrec(i),nctrec(i),lsgrid(i),nspout(i),lcompr(i),i2dmet(i),
     7       iutmzn(i),feast(i),fnorth(i),rnlat0(i),relon0(i),
     8       xlat1(i),xlat2(i),pmap(i),utmhem(i),datum(i),daten(i),
     9       clat0(i),clon0(i),clat1(i),clat2(i)
           msource(i)=0
           iptime(i)=1
           isrcinfo(i)=0
           isrcindv(i)=0
           nsrc(i)=0
c ---      No time zone
           xbtz(i)=-999.
           abtz(i)='UTC+????'

         elseif(dataver.EQ.'2.1             ') then
           read(io)cmodel(i),ver(i),level(i),ibyr(i),ibjul(i),ibhr(i),
     1       ibsec(i),xbtz(i),irlg(i),iavg(i),nsecdt(i),nx(i),ny(i),
     2       dxkm(i),dykm(i),ione(i),xorigkm(i),yorigkm(i),nssta(i),
     3       ibcomp(i),iecomp(i),jbcomp(i),jecomp(i),ibsamp(i),
     4       jbsamp(i),iesamp(i),jesamp(i),meshdn(i),
     5       (nstype(i,n),n=1,8),msource(i),
     6       nrec(i),nctrec(i),lsgrid(i),nspout(i),lcompr(i),i2dmet(i),
     7       iutmzn(i),feast(i),fnorth(i),rnlat0(i),relon0(i),
     8       xlat1(i),xlat2(i),pmap(i),utmhem(i),datum(i),daten(i),
     9       clat0(i),clon0(i),clat1(i),clat2(i)
           iptime(i)=2
           if(itype.eq.2) then
c ---         Source info is not written to VISB.DAT files
              isrcinfo(i)=0
           else
              isrcinfo(i)=1
           endif
           isrcindv(i)=msource(i)
           do k=1,mxsrctype
              nsrc(i)=nsrc(i)+nstype(i,k)
           enddo
c ---      Make time zone string
           call BASRUTC(xbtz(i),abtz(i))

         elseif(dataver.EQ.'2.2             ') then
           read(io)cmodel(i),ver(i),level(i),ibyr(i),ibjul(i),ibhr(i),
     1       ibsec(i),abtz(i),irlg(i),iavg(i),nsecdt(i),nx(i),ny(i),
     2       dxkm(i),dykm(i),ione(i),xorigkm(i),yorigkm(i),nssta(i),
     3       ibcomp(i),iecomp(i),jbcomp(i),jecomp(i),ibsamp(i),
     4       jbsamp(i),iesamp(i),jesamp(i),meshdn(i),
     5       (nstype(i,n),n=1,8),msource(i),
     6       nrec(i),nctrec(i),lsgrid(i),nspout(i),lcompr(i),i2dmet(i),
     7       iutmzn(i),feast(i),fnorth(i),rnlat0(i),relon0(i),
     8       xlat1(i),xlat2(i),pmap(i),utmhem(i),datum(i),daten(i),
     9       clat0(i),clon0(i),clat1(i),clat2(i)
           iptime(i)=2
           if(itype.eq.2) then
c ---         Source info is not written to VISB.DAT files
              isrcinfo(i)=0
           else
              isrcinfo(i)=1
           endif
           isrcindv(i)=msource(i)
           do k=1,mxsrctype
              nsrc(i)=nsrc(i)+nstype(i,k)
           enddo
c ---      Specie-level units are in header
           lspunit(i)=.TRUE.
c ---      Make time zone index
           call UTCBASR(abtz(i),xbtz(i))

         elseif(dataver.EQ.'7.0             ' .OR.
     &          dataver.EQ.'TNG-3.0         ') then
c ---      Dataset 7.0 (TNG-3.0) adds discrete receptor pole ht and
c ---      groups, and explicit number of source-types
c ---      Source number record separate dataset 7.0 (rec NCOM+3a)
c ---      Receptor group names in dataset 7.0 (rec NCOM+6a)
           read(io)cmodel(i),ver(i),level(i),ibyr(i),ibjul(i),ibhr(i),
     1       ibsec(i),abtz(i),irlg(i),iavg(i),nsecdt(i),nx(i),ny(i),
     2       dxkm(i),dykm(i),ione(i),xorigkm(i),yorigkm(i),nssta(i),
     3       ibcomp(i),iecomp(i),jbcomp(i),jecomp(i),ibsamp(i),
     4       jbsamp(i),iesamp(i),jesamp(i),meshdn(i),
     5       nsrctype(i),msource(i),nrec(i),nrgrp(i),
     6       nctrec(i),lsgrid(i),nspout(i),lcompr(i),i2dmet(i),
     7       iutmzn(i),feast(i),fnorth(i),rnlat0(i),relon0(i),
     8       xlat1(i),xlat2(i),pmap(i),utmhem(i),datum(i),daten(i),
     9       clat0(i),clon0(i),clat1(i),clat2(i)
c ---      Source number record separate in dataset 7.0
c ---      (record NCOM+3a)
           if(nsrctype(i).GT.mxsrctype) then
             write(io6,*)'ERROR:  Unexpected number of Source Types'
             write(io6,*)'        Expected : ',mxsrctype
             write(io6,*)'        Found    : ',nsrctype(i)
             stop 'Halted in RDWR1'
           endif
           read(io) (nstype(i,k),k=1,nsrctype(i))
           iptime(i)=2
           if(itype.eq.2) then
c ---         Source info is not written to VISB.DAT files
              isrcinfo(i)=0
           else
              isrcinfo(i)=1
           endif
           isrcindv(i)=msource(i)
           do k=1,mxsrctype
              nsrc(i)=nsrc(i)+nstype(i,k)
           enddo
c ---      Specie-level units are in header
           lspunit(i)=.TRUE.
c ---      Make time zone index
           call UTCBASR(abtz(i),xbtz(i))

         else
           write(io6,*)
           write(io6,*)
           write(io6,*)
           write(io6,*)'ERROR:  Unknown DATASET Version: ',dataver
           stop 'Halted in RDWR1 --- See List File'
         endif
c
c ----------------------------------------------------
c ---    Perform QA checks related to array dimensions
c ----------------------------------------------------
c
c ---    Check met. grid size
         if(nx(i).gt.mxnx.or.ny(i).gt.mxny)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- NX or NY too large for array ',
     1         'dimensions -- File: ',i
            write(io6,*)'NX(i) = ',nx(i),' MXNX = ',mxnx
            write(io6,*)'NY(i) = ',ny(i),' MXNY = ',mxny
            stop 'Halted in RDWR1 --- See List File'
         endif
c
c ---    Check number of species
         if(nspout(i).gt.mxspec)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- No. species too large for array ',
     1         'dimensions -- File: ',i
            write(io6,*)'NSPOUT(i) = ',nspout(i),' MXSPEC = ',mxspec
            stop 'Halted in RDWR1 --- See List File'
         endif
c
c ---    Check number of gridded receptors
         ngridx=meshdn(i)*(iesamp(i)-ibsamp(i))+1
         ngridy=meshdn(i)*(jesamp(i)-jbsamp(i))+1
         if(ngridx.gt.mxnxg.or.ngridy.gt.mxnyg)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many gridded receptors for ',
     1         'array dimensions -- File: ',i
            write(io6,*)'NGRIDX = ',ngridx,' MXNXG = ',mxnxg
            write(io6,*)'IESAMP = ',iesamp(i),' IBSAMP = ',ibsamp(i)
            write(io6,*)'MESHDN = ',meshdn(i)
            write(io6,*)'NGRIDY = ',ngridy,' MXNYG = ',mxnyg
            write(io6,*)'JESAMP = ',jesamp(i),' JBSAMP = ',jbsamp(i)
            stop 'Halted in RDWR1 --- See List File'
         endif
c
c ---    Check number of discrete receptors
         if(nrec(i).gt.mxrec)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many discrete receptors for ',
     1         'array dimensions -- File: ',i
            write(io6,*)'NREC(i) = ',nrec(i),' MXREC = ',mxrec
            stop 'Halted in RDWR1 --- See List File'
         endif 
c
c ---    Check number of discrete receptor-groups (added in 7.0)
         if(nrgrp(i).gt.mxrgrp)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many discrete receptor-groups ',
     1         'for array dimensions -- File: ',i
            write(io6,*)'NRGRP(i) = ',nrgrp(i),' MXRGRP = ',mxrgrp
            stop 'Halted in RDWR1 --- See List File'
         endif 
c
c ---    Check number of complex terrain receptors
         if(nctrec(i).gt.mxrect)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many CTSG receptors for array ',
     1         'dimensions -- File: ',i
            write(io6,*)'NCTREC(i) = ',nctrec(i),' MXRECT = ',mxrect
            stop 'Halted in RDWR1 --- See List File'
         endif                                   
c
c ---    Check number of sources
         if(nsrc(i).gt.mxsrc .AND. isrcinfo(i).NE.0)then
            write(io6,*)
            write(io6,*)
            write(io6,*)
            write(io6,*)'ERROR -- Too many sources for array ',
     1         'dimensions -- File: ',i
            write(io6,*)'NSRC(i) = ',nsrc(i),' MXSRC = ',mxsrc
            stop 'Halted in RDWR1 --- See List File'
         endif 
c
c ---    Number of surface stations should match in all files
         if(i.eq.1)then
c ---       Set output variable NSSTAI
            nsstai=nssta(i)
            if(nsstai.gt.mxss)then
               write(io6,*)
               write(io6,*)
               write(io6,*)
               write(io6,*)'ERROR -- NSSTAI too large for array ',
     1         'dimensions'
               write(io6,*)'NSSTAI = ',nsstai,' MXSS = ',mxss
               stop 'Halted in RDWR1 --- See List File'
            endif
         else
            if(nsstai.ne.nssta(i))then
               write(io6,*)
               write(io6,*)
               write(io6,*)
               write(io6,*)'ERROR in RDWR1 -- NSSTA does not ',
     1         'match expected value'
               write(io6,*)'NSSTAI = ',nsstai,' I = ',i
               write(io6,*)'NSSTA  = ',(nssta(ii),ii=1,i)
               stop 'Halted in RDWR1 --- See List File'
            endif
         endif
c
c ---    HEADER RECORD NCOM+4 -- Run title
         read(io)(title(n,i),n=1,3)
c
c ---    HEADER RECORD NCOM+5 -- List of species output
         nspeci=nspout(i)
         read(io)(csout(n,i),n=1,nspeci)
c ---    Units for each specie/level in list (record NCOM+5a)
c ---    (Introduced in Dataset 2.2)
         if(LSPUNIT(i)) then
            read(io) (aspunit(n,i),n=1,nspeci)
         else
            do n=1,nspeci
               if(csout(n,i)(15:15).EQ.'F') then
                  aspunit(n,i)='g/m2/s          '
               else
                  aspunit(n,i)='g/m3            '
               endif
            enddo
         endif
c
c ---    HEADER RECORD NCOM+6 -- Discrete (non-gridded) receptor data
c ---    (New in dataset 7.0: htrec, irgrp)
         nreci=nrec(i)
         nrgrpi=nrgrp(i)
         if(nreci.gt.0)then
            if(nrgrpi.EQ.0) read(io) (xrec(n,i),n=1,nreci),
     &                               (yrec(n2,i),n2=1,nreci),
     &                               (zrec(n3,i),n3=1,nreci)
            if(nrgrpi.GE.1) then
c ---          (7.0 and up record NCOM+6, 6a)
               read(io) (xrec(n,i),n=1,nreci),
     &                  (yrec(n2,i),n2=1,nreci),
     &                  (zrec(n3,i),n3=1,nreci),
     &                  (htrec(n4,i),n4=1,nreci),
     &                  (irgrp(n4,i),n4=1,nreci)
               read(io) (argrpnam(n,i),n=1,nrgrpi)
            endif
         endif
c
c ---    HEADER RECORD NCOM+7 -- Complex terrain receptor data
         nctreci=nctrec(i)
         if(nctreci.gt.0)then
            read(io)(xrct(n,i),n=1,nctreci),(yrct(n2,i),n2=1,nctreci),
     1       (zrct(n3,i),n3=1,nctreci),(ihill(n4,i),n4=1,nctreci)
         endif


c ---   Dataset version 2.1 introduces source names
c ---   Overwrite names in CSOURCE array (each file same)
        nstot=0
c ---   HEADER RECORDs #NCOM+8 to 15 -- Source names
        if(isrcinfo(i).EQ.1) then
           lfatal=.FALSE.
c ---      Loop over source types
           do k=1,mxsrctype
              if(nstype(i,k).GT.0 .AND. .NOT.LFATAL) then
                 n1=nstot+1
                 nstot=nstot+nstype(i,k)
                 read(io) ktype,(csource(n),n=n1,nstot)
                 if(ktype.NE.k) lfatal=.TRUE.
              endif
           enddo
           if(LFATAL) then
c ---         Report problem information and quit
              write(*,*)
              write(*,*)
              write(*,100)'  *  Unexpected source type found, or     *'
              write(*,100)'  *  Unexpected source number found.      *'
              write(*,100)'  *  Number of sources expected:          *'
              do k=1,mxsrctype
                 write(*,101)'  *     '//cstype(k),nstype(i,k)
              enddo
              write(*,100)'  *  Source Names Read:                   *'
              do k=1,mxsrctype
                 do n=1,nstot
                    write(*,*)cstype(k),csource(n)
                 enddo
              enddo
              stop
           endif
        endif

100     format(1x,a48)
101     format(1x,a12,2(8x,i8),'   *')

c
c ---    HEADER RECORD NCOM+8  -- Nearest Surface Station for VISIB ONLY
c ---                  NCOM+9  -- X coord of stations for VISIB ONLY
c ---                  NCOM+10 -- Y coord of stations for VISIB ONLY
c
c frr (4/02) if 2D arrays of RH, no records in CALPUFF for nearest sf station
c         if(ITYPE.eq.2)then
         if( ITYPE.eq.2 .and. i2dmet(i).eq.0 )then
c
c ---       Nearest station array (Header Record #6)
c ---       Note: re-using NEARS array for each file to avoid a 
c ---             3-D array
            read(io)cnamen,((nears(ii,jj),ii=1,nx(i)),jj=1,ny(i))
c            
c ---       X-coordinate (km) of each surface station (Record #7)
            read(io)cname1(i),(xkmsta(ii,i),ii=1,nssta(i))
c
c ---       Y-coord (km) of each surface station (Record #8)
            read(io)cname2(i),(ykmsta(ii,i),ii=1,nssta(i))
         endif
      enddo


c --- Define additional output variables
      lsamp=lsgrid(1)
c
c --- Sampling grid dimensions
      if(lsamp)then
         nxsam=meshdn(1)*(iesamp(1)-ibsamp(1))+1
         nysam=meshdn(1)*(jesamp(1)-jbsamp(1))+1
      else
         nxsam=0
         nysam=0
      endif
c
c FRR (4/02) Actual met grid size (for 2D array of RH)
      nxmet=nx(1)
      nymet=ny(1)

c --- Recompute the starting date-time if PERIODS are skipped at the
c --- start of the first file
      ibyr1=ibyr(1)
      ibjul1=ibjul(1)
      ibhr1=ibhr(1)
      ibsec1=0
      nhrinc=nskip
      if(iptime(1).EQ.2) then
         ibsec1=ibsec(1)
         nsecavg=iavg(1)*nsecdt(1)
         ibsec1=ibsec1+nskip*nsecavg
         if(ibsec1.GE.3600) then
            nhrinc=ibsec1/3600
            ibsec1=MOD(ibsec1,3600)
         else
            nhrinc=0
         endif
      endif
      if(nhrinc.GT.0) call INCR(io6,ibyr1,ibjul1,ibhr1,nhrinc)

c
c ----------------------------------------------------------------------
c --- Check & write header record data to the list file
c ----------------------------------------------------------------------
c
      do j=1,nfiles
         write(io6,*)
         write(io6,*)' Title lines from file No.: ',j
         do i=1,3
            write(io6,12)title(i,j)
12          format(5x,a80)
         enddo
      enddo
c
      write(io6,*)
      write(io6,*)' Selected header record data'
      write(io6,*)'   CMODEL: ',cmodel(1)
      write(io6,*)'   VER:     ',ver(1)
      write(io6,*)'   LEVEL:   ',level(1)
      write(io6,*)'   IBYR:    ',ibyr1
      write(io6,*)'   IBJUL:   ',ibjul1
      write(io6,*)'   IBHR:    ',ibhr1
      if(iptime(1).EQ.2) then
         write(io6,*)'   IBSEC:   ',ibsec(1)
         write(io6,*)'   NSECDT:  ',nsecdt(1)
         write(io6,*)'   XBTZ:    ',xbtz(1)
         write(io6,*)'   ABTZ:    ',abtz(1)
      endif
      write(io6,*)'   IRLG:    ',ntotal    ! changed Wu (org: irlg(1))
      write(io6,*)'   IAVG:    ',iavg(1)
      write(io6,*)'   PMAP:    ',pmap(1)
      write(io6,*)'   DATUM:   ',datum(1)
      write(io6,*)'   NX:      ',nx(1)
      write(io6,*)'   NY:      ',ny(1)
      write(io6,*)'   DXKM:    ',dxkm(1)
      write(io6,*)'   DYKM:    ',dykm(1)
      write(io6,*)'   IONE:    ',ione(1)
      write(io6,*)'   XORIGKM: ',xorigkm(1)
      write(io6,*)'   YORIGKM: ',yorigkm(1)
      write(io6,*)'   NSSTA:   ',nssta(1)
      write(io6,*)'   IBCOMP:  ',ibcomp(1)
      write(io6,*)'   IECOMP:  ',iecomp(1)
      write(io6,*)'   JBCOMP:  ',jbcomp(1)
      write(io6,*)'   JECOMP:  ',jecomp(1)
      write(io6,*)'   IBSAMP:  ',ibsamp(1)
      write(io6,*)'   IESAMP:  ',iesamp(1)
      write(io6,*)'   JBSAMP:  ',jbsamp(1)
      write(io6,*)'   JESAMP:  ',jesamp(1)
      write(io6,*)'   MESHDN:  ',meshdn(1)
      write(io6,*)'   NREC:    ',nrec(1)
      write(io6,*)'   NCTREC:  ',nctrec(1)
      write(io6,*)'   LSGRID:  ',lsgrid(1)
      write(io6,*)'   LCOMPR:  ',lcompr(1)
      write(io6,*)'   NSPOUT:  ',nspout(1)
      if(isrcinfo(1).EQ.1) then
         write(io6,*)'   MSOURCE: ',msource(1)
      endif
c
      write(io6,1010) cstype
1010  format(/1x,'File ',13(5x,a4))
      do i=1,nfiles
      write(io6,1012)i,(nstype(i,n),n=1,mxsrctype)
1012     format(1x,i4,13(4x,i5))
      enddo
c
      nspeci=nspout(1)
c --- Write species names units to list file (introduced Dataset 2.2)
      write(io6,*)
      write(io6,'(a)')'  SPECIES          UNITS          (file 1)'
      write(io6,'(a)')'---------------  ----------------'
      do n=1,nspeci
         write(io6,'(a15,2x,a16)') csout(n,1),aspunit(n,1)
      enddo

c --- Perform additional QA checks
c --------------------------------

c --- First level checks
      ierr=0
      do i=2,nfiles
         if(nsrc(i).ne.nsrc(1))ierr=ierr+1
         if(nsrctype(i).ne.nsrctype(1))ierr=ierr+1
         if(isrcinfo(i).ne.isrcinfo(1))ierr=ierr+1
         if(isrcindv(i).ne.isrcindv(1))ierr=ierr+1
         if(msource(i).ne.msource(1))ierr=ierr+1
         if(iptime(i).ne.iptime(1))ierr=ierr+1
         do k=1,mxsrctype
            if(nstype(i,k).ne.nstype(1,k))ierr=ierr+1
         enddo
         if(nrgrp(i).ne.nrgrp(1))ierr=ierr+1
         if(nspout(i).ne.nspout(1))ierr=ierr+1
         if(lsgrid(i)) then
            if(.not.lsgrid(1)) ierr=ierr+1
         else
            if(lsgrid(1)) ierr=ierr+1
         endif
         if(lcompr(i)) then
            if(.not.lcompr(1)) ierr=ierr+1
         else
            if(lcompr(1)) ierr=ierr+1
         endif
         if(ABS(xorigkm(i)-xorigkm(1)).gt.0.001)ierr=ierr+1
         if(ABS(yorigkm(i)-yorigkm(1)).gt.0.001)ierr=ierr+1
c ---    Demand that all files have units if any have units
c ---    (this could be relaxed if output dataset is changed)
         if(lspunit(i)) then
            if(.not.lspunit(1)) ierr=ierr+1
         else
            if(lspunit(1)) ierr=ierr+1
         endif
      enddo
      if(ierr.gt.0)then
         write(io6,*)
         write(io6,*)
         write(io6,*)
         write(io6,*)'ERROR -- Header record data do not match!'
         write(io6,*)'IERR    = ',ierr
         write(io6,*)
         write(io6,*)
         write(io6,*)'NSRC    = ',(nsrc(n),n=1,nfiles)
         write(io6,*)'NSRCTYPE= ',(nsrctype(n),n=1,nfiles)
         write(io6,*)'ISRCINFO= ',(isrcinfo(n),n=1,nfiles)
         write(io6,*)'ISRCINDV= ',(isrcindv(n),n=1,nfiles)
         write(io6,*)'MSOURCE = ',(msource(n),n=1,nfiles)
         write(io6,*)'IPTIME  = ',(iptime(n),n=1,nfiles)
         do k=1,mxsrctype
            write(io6,*)cstype(k)//'   = ',(nstype(n,k),n=1,nfiles)
         enddo
         write(io6,*)'NREC    = ',(nrec(n),n=1,nfiles)
         write(io6,*)'NRGRP   = ',(nrgrp(n),n=1,nfiles)
         write(io6,*)'NCTREC  = ',(nctrec(n),n=1,nfiles)
         write(io6,*)'NSPOUT  = ',(nspout(n),n=1,nfiles)
         write(io6,*)'LSGRID  = ',(lsgrid(n),n=1,nfiles)
         write(io6,*)'LCOMPR  = ',(lcompr(n),n=1,nfiles)
         write(io6,*)'XORIGKM = ',(xorigkm(n),n=1,nfiles)
         write(io6,*)'YORIGKM = ',(yorigkm(n),n=1,nfiles)
         write(io6,*)'LSPUNIT = ',(lspunit(n),n=1,nfiles)
         stop 'Halted in RDWR1 --- See List File'
      endif

c --- Second level checks
c --- Demand that species names and units match (same order)
      ierr=0
      do n=2,nfiles
         do k=1,nspout(1)
            if(csout(k,n).NE.csout(k,1))ierr=n
            if(aspunit(k,n).NE.aspunit(k,1))ierr=n
         enddo
      enddo
      if(ierr.gt.0)then
         write(io6,*)
         write(io6,*)
         write(io6,*)
         write(io6,*)'ERROR -- Species names/units do not match!'
         write(io6,*)'Example: File  = ',ierr
         write(io6,*)
         write(io6,*)'   File:    1             ',ierr
         do k=1,nspout(1)
            write(io6,*)csout(k,1),aspunit(k,1),
     &                  csout(k,ierr),aspunit(k,ierr)
         enddo
         stop 'Halted in RDWR1 --- See List File'
      endif

c
c --- Determine if this is a flux file (so there will be no values
c --- at CTSG receptors)
      iflux=0
      if(csout(1,1)(13:15).eq.' DF'.or.csout(1,1)(13:15).eq.' WF')then
         iflux=1
      endif
c
c --- Check for proper variable label with RH/visibility file
      if(itype.eq.2)then
         do i=1,nfiles
            if(csout(1,i)(1:8).ne.' REL HUM')then
              write(io6,*)
              write(io6,*)
              write(io6,*)
              write(io6,*)'ERROR in RDWR1 -- Invalid RH file -- I = ',I
              write(io6,*)'CSOUT = ',csout(1,i)
              stop 'Halted in RDWR1 --- See List File'
            endif
         enddo
      endif
c
c ----------------------------------------------------------------------
c --- OUTPUT of header records for summed file
c
      i2drh=i2dmet(1)
c
c --- Record #1 - File Declaration -- 24 words
      if(dataver.EQ.'TNG-3.0         ') then
c ---    Redirect output dataset to 7.0 in this case
         write(io10) dataset,'7.0             ',datamod
      else
         write(io10) dataset,dataver,datamod
      endif
c --- Record #2 - Number of comment lines -- 1 word
      ncom=ncommout+1
      write(io10) ncom
c --- Record #3 to NCOM+2 (Comment record section) -- 33 words each
      write(io10) doc1
c --- Go to beginning of the scratch file with the control file images
      REWIND(iox)
c --- Loop over records
      do i=1,ncommout
         comment1=blank
         read(iox,'(a132)') comment1
         write(io10) comment1
      enddo
      CLOSE(iox)

c ---    HEADER RECORD NCOM+3 -- General run parameters
c
      if(dataver.EQ.'2.0             ') then
        write(io10)cmodel(1),ver(1),level(1),ibyr1,ibjul1,ibhr1,
     1    ntotal,iavg(1),nx(1),ny(1),dxkm(1),dykm(1),ione(1),
     2    xorigkm(1),yorigkm(1),nssta(1),ibcomp(1),iecomp(1),jbcomp(1),
     3    jecomp(1),ibsamp(1),jbsamp(1),iesamp(1),jesamp(1),meshdn(1),
     4    nrec(1),nctrec(1),
     5    nstype(1,1),nstype(1,3),nstype(1,5),nstype(1,7),
     5    lsgrid(1),nspout(1),lcompr(1),i2dmet(1),
     6    iutmzn(1),feast(1),fnorth(1),rnlat0(1),relon0(1),
     7    xlat1(1),xlat2(1),pmap(1),utmhem(1),datum(1),daten(1),
     8    clat0(1),clon0(1),clat1(1),clat2(1)

      elseif(dataver.EQ.'2.1             ') then
        write(io10)cmodel(1),ver(1),level(1),ibyr1,ibjul1,ibhr1,
     1    ibsec(1),xbtz(1),ntotal,iavg(1),nsecdt(1),nx(1),ny(1),
     2    dxkm(1),dykm(1),ione(1),xorigkm(1),yorigkm(1),nssta(1),
     3    ibcomp(1),iecomp(1),jbcomp(1),jecomp(1),ibsamp(1),
     4    jbsamp(1),iesamp(1),jesamp(1),meshdn(1),
     5    (nstype(1,n),n=1,8),msource(1),
     6    nrec(1),nctrec(1),lsgrid(1),nspout(1),lcompr(1),i2dmet(1),
     7    iutmzn(1),feast(1),fnorth(1),rnlat0(1),relon0(1),
     8    xlat1(1),xlat2(1),pmap(1),utmhem(1),datum(1),daten(1),
     9    clat0(1),clon0(1),clat1(1),clat2(1)

      elseif(dataver.EQ.'2.2             ') then
        write(io10)cmodel(1),ver(1),level(1),ibyr1,ibjul1,ibhr1,
     1    ibsec(1),abtz(1),ntotal,iavg(1),nsecdt(1),nx(1),ny(1),
     2    dxkm(1),dykm(1),ione(1),xorigkm(1),yorigkm(1),nssta(1),
     3    ibcomp(1),iecomp(1),jbcomp(1),jecomp(1),ibsamp(1),
     4    jbsamp(1),iesamp(1),jesamp(1),meshdn(1),
     5    (nstype(1,n),n=1,8),msource(1),
     6    nrec(1),nctrec(1),lsgrid(1),nspout(1),lcompr(1),i2dmet(1),
     7    iutmzn(1),feast(1),fnorth(1),rnlat0(1),relon0(1),
     8    xlat1(1),xlat2(1),pmap(1),utmhem(1),datum(1),daten(1),
     9    clat0(1),clon0(1),clat1(1),clat2(1)

      elseif(dataver.EQ.'7.0             ' .OR.
     &       dataver.EQ.'TNG-3.0         ') then
        write(io10)cmodel(1),ver(1),level(1),ibyr1,ibjul1,ibhr1,
     1    ibsec(1),abtz(1),ntotal,iavg(1),nsecdt(1),nx(1),ny(1),
     2    dxkm(1),dykm(1),ione(1),xorigkm(1),yorigkm(1),nssta(1),
     3    ibcomp(1),iecomp(1),jbcomp(1),jecomp(1),ibsamp(1),
     4    jbsamp(1),iesamp(1),jesamp(1),meshdn(1),nsrctype(1),
     5    msource(1),nrec(1),nrgrp(1),
     6    nctrec(1),lsgrid(1),nspout(1),lcompr(1),i2dmet(1),
     7    iutmzn(1),feast(1),fnorth(1),rnlat0(1),relon0(1),
     8    xlat1(1),xlat2(1),pmap(1),utmhem(1),datum(1),daten(1),
     9    clat0(1),clon0(1),clat1(1),clat2(1)
c ---   HEADER RECORD NCOM+3a -- Number of sources of each type
        write(io10)(nstype(1,k),k=1,nsrctype(1))

      endif
c
c --- HEADER RECORD #NCOM+4 -- Run title
      write(io10)(titleo(n),n=1,3)
c
c --- HEADER RECORD #NCOM+5 -- List of species output
      nspeci=nspout(1)
      write(io10)(csout(n,1),n=1,nspeci)
      if(LSPUNIT(1)) then
c ---    HEADER RECORD #NCOM+5a -- List of species units
         write(io10) (aspunit(n,1),n=1,nspeci)
      endif
c
c --- HEADER RECORD #NCOM+6 -- Discrete (non-gridded) receptor data
c --- (New in dataset 7.0: htrec, irgrp)
      nreci=nrec(1)
      nrgrpi=nrgrp(1)
      if(nreci.gt.0)then
         if(nrgrpi.EQ.0) write(io10)(xrec(n,1),n=1,nreci),
     &                              (yrec(n2,1),n2=1,nreci),
     &                              (zrec(n3,1),n3=1,nreci)
         if(nrgrpi.GE.1) then
c ---       (7.0 and up record NCOM+6, 6a)
            write(io10) (xrec(n,1),n=1,nreci),
     &                  (yrec(n2,1),n2=1,nreci),
     &                  (zrec(n3,1),n3=1,nreci),
     &                  (htrec(n4,1),n4=1,nreci),
     &                  (irgrp(n4,1),n4=1,nreci)
            write(io10) (argrpnam(n,1),n=1,nrgrpi)
         endif
      endif
c
c --- HEADER RECORD #NCOM+7 -- Complex terrain receptor data
      nctreci=nctrec(1)
      if(nctreci.gt.0)then
         write(io10)(xrct(n,1),n=1,nctreci),(yrct(n2,1),n2=1,nctreci),
     1    (zrct(n3,1),n3=1,nctreci),(ihill(n4,1),n4=1,nctreci)
      endif
c
c --- HEADER RECORDs #NCOM+8 to 15 -- Source names
      if(isrcinfo(1).EQ.1) then
c ---    Loop over source types
         nstot=0
         do k=1,mxsrctype
            if(nstype(1,k).GT.0) then
               n1=nstot+1
               nstot=nstot+nstype(1,k)
               write(io10) k,(csource(n),n=n1,nstot)
            endif
         enddo
      endif
c
c --- HEADER RECORD
c ---      #NCOM+8 -- Nearest Surface Station for VISIB ONLY
c ---      #NCOM+9 -- X coord (UTM) of stations for VISIB ONLY
c ---     #NCOM+10 -- Y coord (UTM) of stations for VISIB ONLY
c frr (4/02) if 2D arrays of RH, no records in CALPUFF for nearest sf station
c      if(ITYPE.eq.2)then
      if( ITYPE.eq.2 .and. i2drh.eq.0 )then
c
c ---    Nearest station array (Header Record #NCOM+8)
         write(io10)cnamen,((nears(ii,jj),ii=1,nx(1)),jj=1,ny(1))
c            
c ---    X-coordinate (km) of each surface station (Record #NCOM+9)
         write(io10)cname1(i),(xkmsta(ii,1),ii=1,nssta(1))
c
c ---    Y-coord (km) of each surface station (Record #NCOM+10)
         write(io10)cname2(i),(ykmsta(ii,1),ii=1,nssta(1))
      endif

c --- Pass additional control variables to arg list
      lcompr_=lcompr(1)
      isrcinfo_=isrcinfo(1)
      isrcindv_=isrcindv(1)
      nsrc_=nsrc(1)
      iptime_=iptime(1)

c
      return
      end
c----------------------------------------------------------------------
      subroutine rdwrhr(io,lskip,nspeci,nreci,nctreci,lsamp,
     1                  nxsam,nysam,iflux,lcompr,nfile,itype,isrcinfo,
     2                  isrcindv,nsrc,itime,nsstai,nxmet,nymet,i2drh,
     3                  lend)
c----------------------------------------------------------------------
c
c --- APPEND     Version: 7.0.0      Level:  051122             RDWRHR
c                J. Scire, EARTH TECH
c
c --- PURPOSE:  Read data records of concentration, flux, or RH data 
c               and output to a new CALPUFF file (if not skipping)
c               
c --- UPDATE
c --- V2.34(051122) from V2.3 (031017) (DGS)
c                    - Remove "hour" terminology
c --- V2.3 (031017) from V2.2 (020401) (DGS)
c                    - CALPUFF.DAT dataset version 2.1 format
c                    - Add skip option (RDHR removed)
c
c --- INPUTS:
c                IO - integer       - Unit number of the input CALPUFF 
c                                     file
c             LSKIP - Logical       - Output control flag
c                                     T:  No output (skip)
c                                     F:  Write to file
c            NSPECI - integer       - Number of species in files
c             NRECI - integer       - Number of discrete receptors
c           NCTRECI - integer       - Number of special complex terrain
c             LSAMP - logical       - Flag indicating if gridded
c                                     receptors used (T=yes, F=no)
c                                     (CTSG) receptors
c             NXSAM - integer       - No. cells in X direction of
c                                     gridded receptor grid
c             NYSAM - integer       - No. cells in Y direction of
c                                     gridded receptor grid
c             IFLUX - integer       - Flag indicating if file are
c                                     concentrations (IFLUX=0) or
c                                     wet/dry fluxes (IFLUX=1).  The
c                                     CTSG receptors have no flux data
c            LCOMPR - logical       - Data compression flag
c             NFILE - integer       - Number of the file being 
c                                     processed in this call
c             ITYPE - integer       - File type:
c                                        1 = concentration/flux files
c                                        2 = relative humidity files
c          ISRCINFO - integer       - Source information flag
c          ISRCINDV - integer       - Individual source contribution
c                                     flag
c              NSRC - integer       - Number of sources
c             ITIME - integer       - Time format flag
c            NSSTAI - integer       - Number of surface (RH) stations
c                                     in visibility file
c             NXMET - integer       - Actual x dimension of met grid
c             NYMET - integer       - Actual y dimension of met grid
c             I2DRH - integer       - Flag indicating if 2D array of RH
c
c       Parameters:
c             MXNXG, MXNYG, MXREC, MXRECT, MXSPEC, MXFIL
c             IO6, IO10, IO21
c
c --- OUTPUT:
c              LEND - logical       - Flag indicating if end-of-file
c                                     has been reached
c
c --- RDWRHR called by: MAIN
c --- RDWRHR calls:     RDDAT, RWDAT, RDCOMPRS, RWCOMPRS
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.app'
c
      real chirec(mxrec,mxspec)
      real chict(mxrect,mxspec)
      real tmp7(mxnxny)
      real xwork1(mxnxg,mxnyg),xwork2(mxrec),xwork3(mxrect)
c
      integer nyr,njul,nhr
      integer irhss(mxss)
c
c frr(4/02) 2D arrays of rel. hum.
      integer irh2d(mxnx,mxny)
c
      character*16 cnamsrc
      character*15 cname(mxspec)
c
      logical lsamp,lend,lcompr,lskip
c
c --- Initializations
      isrc=0
      lend=.FALSE.

c --- Branch for RH visibility file type
      if(itype.eq.2)go to 500

c ------------------------------------------
c --- Read 1-period of concentration/flux data
c ------------------------------------------
c
c --- Loop over source contributions
c ----------------------------------
1     isrc=isrc+1
c ---    Date/time data record
         if(itime.EQ.1) then
            read(io,end=999)nyr,njul,nhr
            if(LSKIP) then
               if(isrc.EQ.1) write(io6,*)'Skipping YR,DAY,HR = ',nyr,
     1                       njul,nhr,' in File: ',nfile
            else
               write(io10)nyr,njul,nhr
            endif
         elseif(itime.EQ.2) then
            read(io,end=999) myrb,mjdayb,mhrb,msecb,
     &                       myre,mjdaye,mhre,msece
            if(LSKIP) then
               if(isrc.EQ.1)write(io6,*)'Skipping YRE,DAYE,HRE,SECE = ',
     1                      myre,mjdaye,mhre,msece,' in File: ',nfile
            else
               write(io10) myrb,mjdayb,mhrb,msecb,
     &                     myre,mjdaye,mhre,msece
            endif
         endif
c ---    Source information record (if present)
         if(isrcinfo.EQ.1) then
            read(io,end=999) ktype,knum,cnamsrc,xskm,yskm
            if(.not.LSKIP) write(io10) ktype,knum,cnamsrc,xskm,yskm
         endif
c
c ---    Loop over species-level
         do k=1,nspeci
c
c ---       Gridded receptor concentrations
            if(lsamp)then

c ---          Modified to read compressed calpuff output files
               if(lcompr)then     ! Wu
                  read(io)ii
                  if(LSKIP) then
                     call rdcomprs(xwork1,ii,io)
                  else
                     write(io10)ii
                     call rwcomprs(xwork1,ii,io)
                  endif
               else
                  if(LSKIP) then
                     call rddat(io,nxsam,nysam,cname(k),tmp7)
                  else
                     call rwdat(io,nxsam,nysam,cname(k),tmp7)
                  endif
               endif
            endif
c
c ---       Discrete receptor concentrations
            if(nreci.GT.0) then
               if(lcompr)then    ! Wu
                  read(io)ii
                  if(LSKIP) then
                     call rdcomprs(xwork2,ii,io)
                  else
                     write(io10)ii
                     call rwcomprs(xwork2,ii,io)
                  endif
               else 
                  if(LSKIP) then
                     call rddat(io,nreci,1,cname(k),chirec(1,k))
                  else
                     call rwdat(io,nreci,1,cname(k),chirec(1,k))
                  endif
               endif
            endif
c
c ---       Discrete CTSG receptor concentrations
            if(nctreci.GT.0.0.and.iflux.eq.0)then
               if(lcompr) then    ! Wu
                  read(io)ii
                  if(LSKIP) then
                     call rdcomprs(xwork3,ii,io)
                  else
                     write(io10)ii
                     call rwcomprs(xwork3,ii,io)
                  endif
               else
                  if(LSKIP) then
                     call rddat(io,nctreci,1,cname(k),chict(1,k))
                  else
                     call rwdat(io,nctreci,1,cname(k),chict(1,k))
                  endif
               endif
            endif
         enddo

c --- Loop back for more data if reading source contributions
c --- There should be NSRC+1 (individuals plus total)
      if(isrcindv.GT.0 .AND. isrc.LE.nsrc) goto 1

      return

500   continue
c -------------------------------------------------
c --- Read 1-period relative humidity/visibility data
c -------------------------------------------------
c
c --- Date/time data record
      if(itime.EQ.1) then
         read(io,end=999)nyr,njul,nhr
         if(LSKIP) then
            write(io6,*)'Skipping YR,DAY,HR = ',nyr,njul,nhr,
     1                  ' in File: ',nfile
         else
            write(io10)nyr,njul,nhr
         endif
      elseif(itime.EQ.2) then
         read(io,end=999) myrb,mjdayb,mhrb,msecb,
     &                    myre,mjdaye,mhre,msece
         if(LSKIP) then
            write(io6,*)'Skipping YRE,DAYE,HRE,SECE = ',myre,
     1                   mjdaye,mhre,msece,' in File: ',nfile
         else
            write(io10) myrb,mjdayb,mhrb,msecb,
     &                  myre,mjdaye,mhre,msece
         endif
      endif
c
c --- Read & write a data record
c FRR (4/02) 2D RH arrays for NOOBS version of CALPUFF
      if(i2drh.eq.1) then
         read(io)cname(1),((irh2d(i,j),j=1,nymet),i=1,nxmet)
         if(.not.LSKIP) write(io10)cname(1),((irh2d(i,j),j=1,nymet),
     &                                       i=1,nxmet)
      else
         read(io)cname(1),(irhss(n),n=1,nsstai)
         if(.not.LSKIP) write(io10)cname(1),(irhss(n),n=1,nsstai)
      endif

      return

c --- End of file reached
999   print *,'Reached end of data'
      lend =.TRUE.

      return
      end
c----------------------------------------------------------------------
      subroutine rddat(iounit,nx,ny,cname,outarr)
c----------------------------------------------------------------------
      real outarr(nx,ny)
      character*15 cname
c
      read(iounit)cname,outarr
c
      return
      end
c----------------------------------------------------------------------
      subroutine rwdat(iounit,nx,ny,cname,outarr)
c----------------------------------------------------------------------
      include 'params.app'

      real outarr(nx,ny)
      character*15 cname
c
      read(iounit)cname,outarr
      write(io10)cname,outarr
c
      return
      end
c----------------------------------------------------------------------
      subroutine rwcomprs(xwork,ii,io)
c----------------------------------------------------------------------
c read and write compressed data

      include 'params.app'
      real xwork(ii)
      character*15 clabel
c
c --- Read the compressed data record
      read(io)clabel,xwork
      write(io10)clabel,xwork

      return
      end
c----------------------------------------------------------------------
      subroutine rdcomprs(xwork,ii,io)
c----------------------------------------------------------------------
c read compressed data

      real xwork(ii)
      character*15 clabel
c
c --- Read the compressed data record
      read(io)clabel,xwork

      return
      end
c----------------------------------------------------------------------
      subroutine tfercf(title,ncommout)
c----------------------------------------------------------------------
c
c --- APPEND   Version: 7.0.0        Level: 030402              TFERCF
c ---          D. Strimaitis,    Earth Tech, Inc.
c
c --- PURPOSE: Transfer image of control file to scratch file
c
c --- INPUTS:
c          TITLE(3) - c*80          - Control File Title Lines
c
c        Parameters: iox, io5
c
c --- OUTPUT:
c          NCOMMOUT - integer       - Number of comment records
c
c --- TFERCF called by:  MAIN
c --- TFERCF calls:      none
c----------------------------------------------------------------------
c
      include 'params.app'

      character*80 title(3)
      character*132 aline, blank
      character*33 blank33

c --- Set blanks
      data blank33/'                                 '/
      blank(1:33)=blank33
      blank(34:66)=blank33
      blank(67:99)=blank33
      blank(100:132)=blank33

c --- Reposition to start of input control file
      REWIND(io5)

c --- Initialize number of records written to scratch file
      ncommout=0

c --- Determine number of lines in the control file before the
c --- title lines
      read(io5,*) in1
      read(io5,*) in2
      nlines=2*in2+3

c --- Reposition input control file again
      REWIND(io5)

c --- Open scratch file
      OPEN(iox,status='scratch')

c --- Write the title records first
      do i=1,3
         write(iox,'(a80)') title(i)
         ncommout=ncommout+1
      enddo

c --- Transfer other control file records
      do i=1,nlines
         aline=blank
         read(io5,'(a132)',end=999) aline
         write(iox,'(a132)') aline
         ncommout=ncommout+1
      enddo

c --- Transfer completed
999   close(io5)

      return
      end

c----------------------------------------------------------------------
c --- Individual subroutines from CALUTILS .....
c --- INCR, BASRUTC, UTCBASR, OPEN_ERR
c----------------------------------------------------------------------

c----------------------------------------------------------------------
      subroutine incr(io,iyr,ijul,ihr,nhrinc)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.7.0    Level: 000602                   INCR
c                J. Scire, SRC
c
c --- PURPOSE:  Increment the time and date by "NHRINC" hours
c
c --- UPDATE
c ---               000602  (DGS): add message to "stop"
c ---               980304  (DGS): Allow for a negative "increment" of
c                                  up to 24 hours
c ---               980304  (DGS): Allow for arbitrarily large nhrinc
c
c --- INPUTS:
c       IO     - integer - Unit number for list file output
c       IYR    - integer - Current year
c       IJUL   - integer - Current Julian day
c       IHR    - integer - Current hour (00-23)
c       NHRINC - integer - Time increment (hours)
c
c               NOTE: "NHRINC" must >= -24
c                      Hour is between 00-23
c
c --- OUTPUT:
c       IYR    - integer - Updated year
c       IJUL   - integer - Updated Julian day
c       IHR    - integer - Updated hour (00-23)
c
c --- INCR called by: host subroutines
c --- INCR calls:     none
c----------------------------------------------------------------------
c
c --- Check nhrinc
      if(nhrinc.lt.-24) then
         write(io,*)'ERROR IN SUBR. INCR -- Invalid value of NHRINC ',
     1   '-- NHRINC = ',nhrinc
         write(*,*)
         stop 'Halted in INCR -- see list file.'
      endif

c --- Save increment remaining (needed if nhrinc > 8760)
      nleft=nhrinc
c
c --- Process change in hour
      if(nhrinc.gt.0)then
c
10       ninc=MIN0(nleft,8760)
         nleft=nleft-ninc
c
c ---    Increment time
         ihr=ihr+ninc
         if(ihr.le.23)return
c
c ---    Increment day
         ijul=ijul+ihr/24
         ihr=mod(ihr,24)
c
c ---    ILEAP = 0 (non-leap year) or 1 (leap year)
         if(mod(iyr,4).eq.0)then
            ileap=1
         else
            ileap=0
         endif
c
         if(ijul.gt.365+ileap) then
c ---       Update year
            iyr=iyr+1
            ijul=ijul-(365+ileap)
         endif
c
c ---    Repeat if more hours need to be added
         if(nleft.GT.0) goto 10
c
      elseif(nhrinc.lt.0)then
c ---    Decrement time
         ihr=ihr+nhrinc
         if(ihr.lt.0)then
            ihr=ihr+24
            ijul=ijul-1
            if(ijul.lt.1)then
               iyr=iyr-1
               if(mod(iyr,4).eq.0)then
                  ijul=366
               else
                  ijul=365
               endif
            endif
         endif
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine utcbasr(axtz,xbtz)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.7.0    Level: 070327                UTCBASR
c ---            F.Robe, Earth Tech
c
c --- PURPOSE:  Converts character string UTC time zone 
c               to real base time zone
c
c --- V2.55 (070327) from V2.5 (041123) (DGS)
c               - Add RETURN statement
c
c --- INPUT:
c          AXTZ - char*8    - time zone (international convention: 
c                             relative to UTC/GMT)UTC-HHMM
c --- OUTPUT:
c          XBTZ - real      - base time zone (old convention: positive
c                             in North America i.e. opposite to UTC)
c
c --- UTCBASR called by:  host subroutines
c --- UTCBASR calls:      none
c----------------------------------------------------------------------
      character*8 axtz

      read(axtz(4:6),'(i3)')ihr
      read(axtz(7:8),'(i2)')imin
      if(ihr.lt.0)imin=-imin

      xbtz=ihr+imin/60.

c --- Flip sign as base time convention is opposite UTC/GMT
      xbtz=-xbtz

      return
      end
c----------------------------------------------------------------------
      subroutine basrutc(xbtz,axtz)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.7.0    Level: 070327                BASRUTC
c ---            F.Robe, Earth Tech
c
c --- PURPOSE:  Converts real base time zone  to character string
c               UTC time zone
c
c --- UPDATE
c --- V2.55 (070327) from V2.5 (041123) (DGS)
c               - Fix output format of time zone string for zone=0
c               - Add RETURN statement
c
c --- INPUT:
c          XBTZ - real      - base time zone (old convention: positive
c                             in North America i.e. opposite to UTC)

c --- OUTPUT:
c          AXTZ - real      - time zone (international convention: 
c                             relative to UTC/GMT)UTC-HHMM
c
c --- BASRUTC called by:  host subroutines
c --- BASRUTC calls:      none
c----------------------------------------------------------------------
      character*8 axtz

      ixbtz=int(xbtz) 
c     convert fractional real to minutes
      imin=(xbtz-ixbtz)*60
      ixbtz=ixbtz*100+imin

c --- Define time as "UTC-HHMM" (hours/minutes)
      axtz(1:3)="UTC"

c --- Flip sign as base time zone is minus UTC zone
      if (xbtz.gt.0.) then
         axtz(4:4)="-"
      else
         axtz(4:4)="+"
      endif
c --- Make sure time zone is written as 4 digits
      write(axtz(5:8),'(i4.4)')abs(ixbtz)

      return
      end
c----------------------------------------------------------------------
      subroutine open_err(iolst,cfrom,cftype,cfname,iunit)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.7.0    Level: 141010              OPEN_ERR
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Report error in opening a file
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c                                   (<0 if not available)
c               CFROM - char*     - Called-From string to report error
c              CFTYPE - char*     - File-type string
c              CFNAME - char*     - File-name string
c               IUNIT - integer   - File unit number
c
c --- OUTPUT:
c
c --- OPEN_ERR called by:  ()
c --- OPEN_ERR calls:      
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: cfrom,cftype,cfname
      integer          :: iolst, iunit

      if(iolst.GT.0) then
         write(iolst,*)
         write(iolst,*)'ERROR opening '//TRIM(cftype)
         write(iolst,*)'   File Name: '//TRIM(cfname)
         write(iolst,*)'   File Unit: ',iunit
         write(iolst,*)'Problem reported from '//TRIM(cfrom)
         write(iolst,*)
         write(iolst,*)'The file may not exist in this location'
         write(iolst,*)'Check the spelling of the name and the location'
         write(*,*)
         stop 'ERROR: File not found -- see list file'
      else
         write(*,*)
         write(*,*)'ERROR opening '//TRIM(cftype)
         write(*,*)'   File Name: '//TRIM(cfname)
         write(*,*)'   File Unit: ',iunit
         write(*,*)'Problem reported from '//TRIM(cfrom)
         write(*,*)
         write(*,*)'The file may not exist in this location'
         write(*,*)'Check the spelling of the name, and the location'
         stop
      endif

      end

